#include <propeller.h>
#include <stdlib.h>
#include <stdint.h>
#include <stdio.h>
#include "Game\PinballHSM.h"
#include "Domain\SchedulerRegistry.h"
#include "Lamps\LampShow.h"
#include "Domain\PlayfieldSwitchEnum.h"
#include "CogC_Drivers\IODriver\IODriverMailbox.h"
#include "libpropeller\c++allocate\c++allocate.h"

#define MAX_LOOP_COUNTER 1

extern "C" void __cxa_pure_virtual() {
  while (1);
}

 extern _Driver _SimpleSerialDriver;
//extern _Driver _FullDuplexSerialDriver;
extern _Driver _FileDriver;

/* Driver list */
_Driver *_driverlist[] = {
    &_SimpleSerialDriver,
    &_FileDriver,
    NULL
};

/*
 * This is the structure which we'll pass to the C cog.
 * It contains a small stack for working area, and the
 * mailbox which we use to communicate. See toggle.h
 * for the definition/
 */
static struct {
    uint32_t ioDriverStack[IO_DRIVER_STACK_SIZE];
    volatile ioDriverMailbox_t ioDriverMailbox;
} ioDriverPar;

// Global variables / objects
uint32_t _loopArray[MAX_LOOP_COUNTER];
uint8_t _loopArrayCounter = 0;
uint8_t switchKey = 0;
const uint8_t activated = 128;
SchedulerRegistry schedulerRegistry;
PinballHSM game;

// THIS IS THE PROBLEMATIC PART:
const uint32_t testIndex = 5000;
volatile uint8_t testArr[testIndex];

// Forward declaration.
void initializeScheduler();
void initializeEventNotificationMap();
void handleSwitches(uint8_t);
void playLampShow();

// Scheduler cog stack declaration
static uint8_t schedulerCogStack[EXTRA_STACK_BYTES + 500];       // allocate space for the stack - you need to set up a
                                                                  // stack for each Cog you plan on starting

/*
 * Function to start up a new cog running the IO driver
 * code (which we have placed in the IODriver.cog section)
 */
uint8_t startIODriver(volatile void *parptr) {
  extern unsigned int _load_start_IODriver_cog[];
  #if defined(__PROPELLER_XMM__) || defined(__PROPELLER_XMMC__)
    uint32_t hub_buffer[496];
    extern unsigned int _load_stop_IODriver_cog[];
    memcpy(hub_buffer, _load_start_IODriver_cog, (_load_stop_IODriver_cog - _load_start_IODriver_cog) * sizeof(uint32_t));
    return cognew(hub_buffer, parptr);
  #else
    return cognew(_load_start_IODriver_cog, parptr);
  #endif
}

void schedulerCog(void* par) {
  // Schedule tasks.
  waitcnt(CLKFREQ + CNT);
  while(1) {
    waitcnt((CLKFREQ / 4) + CNT);
    schedulerRegistry.schedule();
    printf("Scheduler COG running...\n");
  }
}

int main (void) {
  // Start the IO driver cog.
  waitcnt(CLKFREQ + CNT);

  startIODriver(&ioDriverPar.ioDriverMailbox);
  cogstart(schedulerCog, NULL, schedulerCogStack, sizeof(schedulerCogStack));

  for (int y = 0; y < testIndex; y++) {
    testArr[y] = 1;
  }

  LampShow lampShow((uint8_t*) ioDriverPar.ioDriverMailbox.lampState, (uint8_t*) ioDriverPar.ioDriverMailbox.outputPort);
  game.init();
  schedulerRegistry.addScheduler(lampShow, 31);

  uint8_t previousSwitchInpurtPort[64];
  for (uint8_t loopCounter = 0; loopCounter < 64; loopCounter++) {
    // Initialize with value 2 which means, not closed and not open.
    previousSwitchInpurtPort[loopCounter] = 2;
  }

  int startCnt = CNT;
  int endCnt = CNT;
  // lampShow.playLampShow(LampShow::Sequence::BallPlunged);

  while (1) {
    startCnt = CNT;
    // Handle only switch change events.
    for (uint8_t i = 0; i < 64; i++) {
      if (previousSwitchInpurtPort[i] == ioDriverPar.ioDriverMailbox.inputPort[i]) {
        // No change happened.
        continue;
      }

      // Switch state changed, notify game state machine.
      previousSwitchInpurtPort[i] = ioDriverPar.ioDriverMailbox.inputPort[i];
      switchKey = i;

      if (ioDriverPar.ioDriverMailbox.inputPort[i] > 0) {   // Switch is activated.
        switchKey = i | activated;
      }

      // Now call the appropriate switch function.
      handleSwitches(switchKey);
    }

    endCnt = CNT;
  }
}

void handleSwitches(uint8_t switchKey) {
  switch (switchKey) {
    case (int)PlayfieldSwitch::ShooterLanePlungerSwitch :
      game.onShooterLanePlungerSwitchClosed();
      break;

    case (int)PlayfieldSwitch::ShooterLanePlungerSwitch | activated :
      game.onShooterLanePlungerSwitchOpened();
      break;

    case (int)PlayfieldSwitch::ShooterLaneRampSwitch :
      game.onShooterLaneRampSwitchClosed();
      break;

    case (int)PlayfieldSwitch::ShooterLaneRampSwitch | activated :
      game.onShooterLaneRampSwitchOpened();
      break;

    case (int)PlayfieldSwitch::OuterLoopRightSwitch :
      game.onOuterLoopRightSwitchClosed();
      break;

    case (int)PlayfieldSwitch::OuterLoopRightSwitch | activated :
      game.onOuterLoopRightSwitchOpened();
      break;

    case (int)PlayfieldSwitch::OuterLoopLeftSwitch :
      game.onOuterLoopLeftSwitchClosed();
      break;

    case (int)PlayfieldSwitch::OuterLoopLeftSwitch | activated :
      game.onOuterLoopLeftSwitchOpened();
      break;

    case (int)PlayfieldSwitch::ThingRampSwitch :
      game.onThingRampSwitchClosed();
      break;

    case (int)PlayfieldSwitch::ThingRampSwitch | activated :
      game.onThingRampSwitchOpened();
      break;

    case (int)PlayfieldSwitch::ThingHoleSwitch :
      game.onThingHoleSwitchClosed();
      break;

    case (int)PlayfieldSwitch::ThingHoleSwitch | activated :
      game.onThingHoleSwitchOpened();
      break;

    case (int)PlayfieldSwitch::FarLeftInlaneSwitch :
      game.onFarLeftInlaneSwitchClosed();
      break;

    case (int)PlayfieldSwitch::FarLeftInlaneSwitch | activated :
      game.onFarLeftInlaneSwitchOpened();
      break;

    case (int)PlayfieldSwitch::LeftOutlaneSwitch :
      game.onLeftOutlaneSwitchClosed();
      break;

    case (int)PlayfieldSwitch::LeftOutlaneSwitch | activated :
      game.onLeftOutlaneSwitchOpened();
      break;

    case (int)PlayfieldSwitch::InnerLeftInlaneSwitch :
      game.onLeftInnerInlaneSwitchClosed();
      break;

    case (int)PlayfieldSwitch::InnerLeftInlaneSwitch | activated :
      game.onLeftInnerInlaneSwitchOpened();
      break;

    case (int)PlayfieldSwitch::RightInlaneSwitch :
      game.onRightInlaneSwitchClosed();
      break;

    case (int)PlayfieldSwitch::RightInlaneSwitch | activated :
      game.onRightInlaneSwitchOpened();
      break;

    case (int)PlayfieldSwitch::RightOutlaneSwitch :
      game.onRightOutlaneSwitchClosed();
      break;

    case (int)PlayfieldSwitch::RightOutlaneSwitch | activated :
      game.onRightOutlaneSwitchOpened();
      break;

    case (int)PlayfieldSwitch::Bumper1Switch :
      game.onBumper1SwitchOpened();
      break;

    case (int)PlayfieldSwitch::Bumper1Switch | activated :
      game.onBumper1SwitchOpened();
      break;

    case (int)PlayfieldSwitch::Bumper2Switch :
      game.onBumper2SwitchClosed();
      break;

    case (int)PlayfieldSwitch::Bumper2Switch | activated :
      game.onBumper2SwitchOpened();
      break;

    case (int)PlayfieldSwitch::Bumper3Switch :
      game.onBumper3SwitchClosed();
      break;

    case (int)PlayfieldSwitch::Bumper3Switch | activated :
      game.onBumper3SwitchOpened();
      break;

    case (int)PlayfieldSwitch::Bumper4Switch :
      game.onBumper4SwitchClosed();
      break;

    case (int)PlayfieldSwitch::Bumper4Switch | activated :
      game.onBumper4SwitchOpened();
      break;

    case (int)PlayfieldSwitch::Bumper5Switch :
      game.onBumper5SwitchClosed();
      break;

    case (int)PlayfieldSwitch::Bumper5Switch | activated :
      game.onBumper5SwitchOpened();
      break;

    case (int)PlayfieldSwitch::LeftFlipperButtonSwitch :
      game.onLeftFlipperButtonSwitchClosed();
      break;

    case (int)PlayfieldSwitch::LeftFlipperButtonSwitch | activated :
      game.onLeftFlipperButtonSwitchOpened();
      break;

    case (int)PlayfieldSwitch::RightFlipperButtonSwitch :
      game.onRightFlipperButtonSwitchClosed();
      break;

    case (int)PlayfieldSwitch::RightFlipperButtonSwitch | activated :
      game.onRightFlipperButtonSwitchOpened();
      break;
  }
}
